import json
import logging
from typing import Any

from gpt_api_client import GPTClient


class GPTTextSummarizerResult:
    def __init__(self, response_object, summarized_text):
        self.response_object = response_object
        self.summarized_text = summarized_text


class GPTTextSummarizer:
    def __init__(self, gpt_client: GPTClient, summary_length_limit_enabled: bool = False,
                 summary_length_unit: str = "sentence", summary_length_value: int = 10):
        self.gpt_client = gpt_client
        self.summary_length_limit_enabled = summary_length_limit_enabled
        self.summary_length_unit = summary_length_unit
        self.summary_length_value = summary_length_value

        logging.info("GPT API client params:")
        logging.info(gpt_client)

        logging.info("Prompt used for text summarization:")
        logging.info(self._format_prompt("[input text]"))

    def _format_prompt(self, input_text: str) -> str:
        if self.summary_length_limit_enabled:
            prompt = f"You are a helpful assistant that summarizes the following text in {self.summary_length_value} {self.summary_length_unit} max. "
        else:
            prompt = "You are a helpful assistant that summarizes the following text. "
        prompt += "If you can't summarize, you must answer \"Unknown.\".\n\n"
        prompt += "Text: "
        prompt += json.dumps(input_text)
        prompt += "\n\nSummary:"
        return prompt

    def get_response(self, input_text: Any) -> GPTTextSummarizerResult:
        prompt = self._format_prompt(str(input_text))
        gpt_response = self.gpt_client.send_prompt(prompt)
        return GPTTextSummarizerResult(gpt_response.raw_response, gpt_response.response_text)
